from typing import Dict, TypeVar

A = TypeVar("A")
B = TypeVar("B")


def reversely(__obj: Dict[A, B]) -> Dict[B, A]:
    """Definitely reversed KV-mapping of a dictionary.

    Examples
    --------

    .. code-block:: python

        data = {"password": 1234, "users": 100_000}
        rdata = reversely(data)
        assert rdata == {1234: "password", 100_000: "users"}

        reveal_type(data)   # dict[str, int]
        reveal_type(rdata)  # dict[int, str]

    Args:
        __obj (dict[A, B]): The dictionary to be reversed.

    Returns:
        dict[B, A]: The reversed dictionary.
    """
    result: Dict[B, A] = {}
    for k, v in __obj.items():
        result.update({v: k})

    return result



from typing import Any, Type, TypeVar

try:
    from typing import TypeGuard
except ImportError:
    from typing_extensions import TypeGuard  # pip install typing-extensions

T = TypeVar("T")


def definitely(_obj: Any, _type: Type[T]) -> TypeGuard[T]:
    """Definitely typed.

    Forces any type on an object.

    Example
    -------

    .. code-block:: python

        name = "Fernando Miguel"
        assert definitely(name, int)

        reveal_type(definitely)
        #           ^^^^^^^^^^
        # runtime: str
        # TYPE_CHECKING: int

    Args:
        _obj (Any): The object to be force-typed.
        _type (type[T]): Any type to write.
    """
    return True



import asyncio
from typing import (
    AsyncIterator,
    Awaitable,
    Callable,
    Iterator,
    ParamSpec,
    TypeVar,
)

T = TypeVar("T")
P = ParamSpec("P")


def asyncily(fn: Callable[P, T]) -> Callable[P, Awaitable[T]]:
    """Run a function asynchronously.

    Examples
    --------

    You can use this as a decorator, and use it outside:

    .. code-block:: python

        @asyncily
        async def duplicate(object: str):
            return object * 2

        await duplicate("Money")  # "MoneyMoney"

    ...or treat it as a "async function factory":

    .. code-block:: python

        def greet(name: str):
            return "Hello, " + name + "!"

        agreet = asyncily(greet)
        await agreet("Logan Paul")  # "Hello, Logan Paul!"

    Args:
        fn (Callable[P, T]): The function to be asyncified.

    Returns:
        Callable[P, Awaitable[T]]: Wrapper.
    """

    async def wrapper(*args: P.args, **kwargs: P.kwargs) -> T:
        return await asyncio.to_thread(fn, *args, **kwargs)

    return wrapper


async def asyncily_iterate(it: Iterator[T]) -> AsyncIterator[T]:
    """Iterate through an iterator asynchronously.

    Examples
    --------

    .. code-block:: python

        def get_resources():
            yield from ["banana", "guava", "apple"]

        async for resource in asyncily_iterate(get_resources()):
            print(resource)

        # Output:
        # banana
        # guava
        # apple

    Args:
        it (Iterator[T]): The iterator to be asyncified.

    Returns:
        AsyncIterator[T]: The asyncified iterator.
    """

    class Done(Exception): ...

    while True:

        def getnext() -> T:
            try:
                return it.__next__()
            except StopIteration:
                raise Done("done.")

        try:
            yield await asyncio.to_thread(getnext)
        except Done:
            break
